package auction.dao;

import javax.inject.Inject;
import javax.persistence.*;
import java.util.*;
import java.io.Serializable;
import java.lang.reflect.ParameterizedType;

import auction.dao.GenericDAO;
import org.hibernate.Session;

/**
 * Implements the generic CRUD data access operations using Java Persistence APIs.
 * <p/>
 * To write a DAO, subclass and parameterize this class with your entity.
 * Of course, assuming that you have a traditional 1:1 appraoch for
 * Entity:DAO design. This is actually an implementation that uses some
 * extensions for Java Persistence from Hibernate - you can see how the
 * packages for the extensions are not imported, but named inline.
 *
 * @author Lamon Gray
 */
public abstract class GenericDAOImpl<T, ID extends Serializable>
		implements GenericDAO<T, ID> {

	private Class<T> entityBeanType;

	protected EntityManager em;

	@SuppressWarnings("unchecked")
	public GenericDAOImpl() {
		this.entityBeanType = (Class<T>) ((ParameterizedType) getClass()
				.getGenericSuperclass()).getActualTypeArguments()[0];
	}

	@Inject
	public void setEntityManager(EntityManager em) {
		this.em = em;
	}

	public Class<T> getEntityBeanType() {
		return entityBeanType;
	}

	public T findById(ID id, boolean lock) {
		T entity;
		if (lock) {
			entity = em.find(getEntityBeanType(), id);
			em.lock(entity, javax.persistence.LockModeType.WRITE);
		} else {
			entity = em.find(getEntityBeanType(), id);
		}
		return entity;
	}

	@SuppressWarnings("unchecked")
	public List<T> findAll() {
		return em.createQuery("from " + getEntityBeanType().getName()).getResultList();
	}


	@SuppressWarnings("unchecked")
	public List<T> findByExample(T exampleInstance, String... excludeProperty) {
		// Using Hibernate, more difficult with EntityManager and EJB-QL
		org.hibernate.Criteria crit = ((org.hibernate.ejb.HibernateEntityManager) em)
				.getSession()
				.createCriteria(getEntityBeanType());
		org.hibernate.criterion.Example example =
				org.hibernate.criterion.Example.create(exampleInstance);
		for (String exclude : excludeProperty) {
			example.excludeProperty(exclude);
		}
		crit.add(example);
		return crit.list();
	}

	public void persistent(T entity) {
		em.persist(entity);
	}

	public T merge(T entity) {
		return em.merge(entity);
	}

	public void remove(T entity) {
		em.remove(entity);
	}

	public void flush() {
		em.flush();
	}

	public void clear() {
		em.clear();
	}

	/**
	 * Use this inside subclasses as a convenience method.
	 */
	@SuppressWarnings("unchecked")
	protected List<T> findByCriteria(org.hibernate.criterion.Criterion... criterion) {
		// Using Hibernate, more difficult with EntityManager and EJB-QL
		org.hibernate.Session session = (Session) em.getDelegate();
		org.hibernate.Criteria crit
				= session.createCriteria(getEntityBeanType());
		for (org.hibernate.criterion.Criterion c : criterion) {
			crit.add(c);
		}
		return crit.list();
	}

	/**
	 * Simulates the Hibernate unique result behaviour. If no result is found,
	 * null is returned instead of throwing an exception. Only if more than one
	 * result is found a {@link NonUniqueResultException} is thrown.
	 *
	 * @param list - a result list
	 * @return either null or the single found entry
	 */
	protected T returnSingleOrNull(List<T> list) {
		if (list.size() == 1)
			return list.get(0);
		else if (list.size() == 0)
			return null;
		else
			throw new NonUniqueResultException("The query did return multiple entries");
	}
}

