package auction.model;

import javax.persistence.Embeddable;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Currency;

/**
 * Represents a monetary amount as value and currency.
 * <p>
 * Has some illustrative and non-funtional money conversion magic.
 * @author Gavin King
 * @author Lamon Gray
 */

//@Embeddable
public class MonetaryAmount implements Serializable {


	private static final long serialVersionUID = 8134834978476987919L;
	
	private BigDecimal value;
	private Currency currency;

	protected MonetaryAmount() {
	}

	public MonetaryAmount(BigDecimal value, Currency currency) {
		this.value = value;
		this.currency = currency;
	}

	public Currency getCurrency() {
		return currency;
	}

	public BigDecimal getValue() {
		return value;
	}

	// ********************** Common Methods ********************** //

	public boolean equals(Object o) {
		if (this == o) return true;
		if (!(o instanceof MonetaryAmount)) return false;

		final MonetaryAmount monetaryAmount = (MonetaryAmount) o;

		if (!currency.equals(monetaryAmount.currency)) return false;
		if (!value.equals(monetaryAmount.value)) return false;

		return true;
	}

	public int hashCode() {
		int result;
		result = value.hashCode();
		result = 29 * result + currency.hashCode();
		return result;
	}

	public String toString() {
		return "Value: '" + getValue() + "', " +
		        "Currency: '" + getCurrency() + "'";
	}

	/**
	 * Compares this amount with the other. It is a fake implementation which compares the value but ignores the currency
	 * @param other - another monetary amount
	 * @return true, if this amount is greater than the other
	 */
	public boolean greaterThen(MonetaryAmount other){
		return value.compareTo(other.value) > 0;
	}


	// ********************** Business Methods ********************** //

	public static MonetaryAmount fromString(String amount, String currencyCode) {
		return new MonetaryAmount(new BigDecimal(amount),
								  Currency.getInstance(currencyCode));
	}

	/**
	 * Converts one amount into another currency.
	 * This method assumes that all currencies are equally valued.
	 *
	 * @param amount - an amount
	 * @param toConcurrency - the target currency
	 * @return the amount converted to the target currency
	 */
	public static MonetaryAmount convert(MonetaryAmount amount,
										 Currency toConcurrency) {
		return new MonetaryAmount(amount.getValue(), toConcurrency);
	}

}
