package auction.service;

import auction.model.*;
import auction.test.DatabaseSetup;
import org.dbunit.operation.DatabaseOperation;
import org.hibernate.cfg.AnnotationConfiguration;
import org.hibernate.ejb.Ejb3Configuration;
import org.hibernate.tool.hbm2ddl.SchemaExport;
import org.jboss.weld.environment.se.Weld;
import org.jboss.weld.environment.se.WeldContainer;
import org.jboss.weld.environment.se.events.ContainerInitialized;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.*;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

/**
 * @author Sebastian Hennebrueder
 */
public class MappingTest {

	private static final String OLD_STREET = "Old street";
	private static final String HIGHWAY_3 = "Highway 3";
	private static final String BAD_VIBEL = "Bad Vibel";
	private static final String SAN_FRANCISCO = "San Francisco";

	final Logger logger = LoggerFactory.getLogger(MappingTest.class);

	private static WeldContainer weldContainer;

	@BeforeClass
	public static void setup() {

		// start the container first to let the Hibernate generate the tables
		weldContainer = new Weld().initialize();
		weldContainer.event().select(ContainerInitialized.class).fire(new ContainerInitialized());


		DatabaseSetup setup = new DatabaseSetup();
		setup.setDataFile("auction/service/basedata.xml").setup();

	}

	@Test
	public void persistAddress() {
		final UserService userService = weldContainer.instance().select(UserService.class).get();
		User holger = new User("Holger", "Foo", "holger", "x", "holger@example.com");
		holger.setHomeAddress(new Address(OLD_STREET, "12345", BAD_VIBEL));
		holger.setBillingAddress(new Address(HIGHWAY_3, "12345", SAN_FRANCISCO));
		userService.save(holger);
		EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
				EntityManagerFactory.class).get();

		EntityManager em = entityManagerFactory.createEntityManager();
		Object[] result = (Object[])
				em.createNativeQuery(
						"select username, home_street, home_city, billing_street, billing_city from users where USER_ID = :id")
						.setParameter("id", holger.getId())
						.getSingleResult();
		em.close();
		assertEquals("holger", result[0]);
		assertEquals(OLD_STREET, result[1]);
		assertEquals(BAD_VIBEL, result[2]);
		assertEquals(HIGHWAY_3, result[3]);
		assertEquals(SAN_FRANCISCO, result[4]);


	}

	@Test
	public void persistSimpleUserType() {
		final AuctionService auctionService = weldContainer.instance().select(AuctionService.class).get();
		final UserService userService = weldContainer.instance().select(UserService.class).get();

		User bidder = userService.findByIdInitializeItems(1L, false);
		User seller = userService.findByIdInitializeItems(2L, false);
		final Calendar calendar = Calendar.getInstance();
		final Date today = calendar.getTime();
		calendar.add(Calendar.DATE, 7);
		final Date nextWeek = calendar.getTime();
		Item item = new Item("A book", "A really nice book", seller, new MonetaryAmount(new BigDecimal(5L), Currency.getInstance("USD")), today, nextWeek);
		auctionService.create(item);
		Bid bid = new Bid(new MonetaryAmount(new BigDecimal(20L), Currency.getInstance("USD")), item, bidder);
		auctionService.placeBid(bid);

		EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
				EntityManagerFactory.class).get();

		EntityManager em = entityManagerFactory.createEntityManager();
		assertNotNull(item.getId());
		List<Bid> bids = em.createQuery("select b from Bid b where b.item.id = :id").setParameter("id", item.getId()).getResultList();
		assertEquals(1, bids.size());
		assertEquals(new BigDecimal(20L), bids.get(0).getAmount().getValue());
		assertEquals(Currency.getInstance("USD"), bids.get(0).getAmount().getCurrency());



	}

	@Test
	public void useCompositeUserType() {
		// set up some data first
		final AuctionService auctionService = weldContainer.instance().select(AuctionService.class).get();
		final UserService userService = weldContainer.instance().select(UserService.class).get();

		User bidder = userService.findByIdInitializeItems(1L, false);
		User seller = userService.findByIdInitializeItems(2L, false);
		final Calendar calendar = Calendar.getInstance();
		final Date today = calendar.getTime();
		calendar.add(Calendar.DATE, 7);
		final Date nextWeek = calendar.getTime();
		Item item = new Item("A book", "A really nice book", seller, new MonetaryAmount(new BigDecimal(5L), Currency.getInstance("USD")), today, nextWeek);
		auctionService.create(item);
		Bid bid = new Bid(new MonetaryAmount(new BigDecimal(33L), Currency.getInstance("USD")), item, bidder);
		auctionService.placeBid(bid);

		// Below we try to query for amount values bigger than 10

		logger.debug("About to query for bids higher than 10");
		EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
				EntityManagerFactory.class).get();

		EntityManager em = entityManagerFactory.createEntityManager();
		assertNotNull(item.getId());
		List<Bid> bids = em.createQuery("select b from Bid b where b.amount.value > 10").getResultList();
		assertTrue(bids.size()>0);


	}

}
