package auction.service;

import auction.model.*;
import org.jboss.weld.environment.se.Weld;
import org.jboss.weld.environment.se.WeldContainer;
import org.jboss.weld.environment.se.events.ContainerInitialized;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * @author Sebastian Hennebrueder
 */
public class MappingTest {

	private static final String OLD_STREET = "Old street";
	private static final String HIGHWAY_3 = "Highway 3";
	private static final String BAD_VIBEL = "Bad Vibel";
	private static final String SAN_FRANCISCO = "San Francisco";

	final Logger logger = LoggerFactory.getLogger(MappingTest.class);

	private static WeldContainer weldContainer;

	private static Date today ;
	private static Date nextWeek;
	@BeforeClass
	public static void setup() {
		final Calendar calendar = Calendar.getInstance();
		today = calendar.getTime();
		calendar.add(Calendar.DATE, 7);
		nextWeek = calendar.getTime();

		// start the container first to let the Hibernate generate the tables
		weldContainer = new Weld().initialize();
		weldContainer.event().select(ContainerInitialized.class).fire(new ContainerInitialized());

	}

	@Test
	public void itemsForSale() {
		UserService userService = weldContainer.instance().select(UserService.class).get();
		AuctionService auctionService = weldContainer.instance().select(AuctionService.class).get();

		User seller = new User("Davorka", "Apple", "davorka", "p", "davorka@example.com");
		userService.save(seller);

		Item item = new Item("Apple", "My computer", seller, today, nextWeek);

		auctionService.create(item);

		EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
				EntityManagerFactory.class).get();

		EntityManager em = entityManagerFactory.createEntityManager();
		final List<Item> items = em.createQuery("select i from Item i where i.seller.id = :id", Item.class)
				.setParameter("id", seller.getId()).getResultList();
		Assert.assertEquals(1, items.size());
		Item reloaded = items.get(0);
		Assert.assertEquals("Apple", reloaded.getName());
	}

	@Test
	public void createItemSellerTest() {
		UserService userService = weldContainer.instance().select(UserService.class).get();
		AuctionService auctionService = weldContainer.instance().select(AuctionService.class).get();

		User seller = new User("Davorka", "IT-Girl", "davorkaIt", "p", "davorka@example.com");
		userService.save(seller);

		Item item = new Item();
		item.setName("Strawberry");
		item.setDescription("My computer");
		item.setStartDate(today);
		item.setEndDate(nextWeek);

		seller.getItemsForSale().add(item);
		item.setSeller(seller);
		auctionService.create(item);

	}

	@Test
	public void addItemToCategoryTest() {
		UserService userService = weldContainer.instance().select(UserService.class).get();
		AuctionService auctionService = weldContainer.instance().select(AuctionService.class).get();

		User seller = new User("Sascha", "Manager", "sascha", "p", "sascha@example.com");
		userService.save(seller);


		Item item = new Item("Strawberry", "A red fruit", seller, today, nextWeek);
		auctionService.create(item);

		Category category = new Category("Fruits");
		category.addItem(item);
		auctionService.create(category);

		EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
				EntityManagerFactory.class).get();
		EntityManager em = entityManagerFactory.createEntityManager();
		final List<Item> items = em.createQuery("select i from Item i  join i.categories c where c.id = :catId", Item.class)
				.setParameter("catId", category.getId()).getResultList();
		Assert.assertEquals(1, items.size());
		Item reloaded = items.get(0);
		Assert.assertEquals("Strawberry", reloaded.getName());

	}

	@Test
	public void deleteItemTest(){
		  // create some data first
		UserService userService = weldContainer.instance().select(UserService.class).get();
		AuctionService auctionService = weldContainer.instance().select(AuctionService.class).get();

		User seller = new User("Holger", "Pomolog", "holger", "p", "holger@example.com");
		userService.save(seller);
		Item item = new Item("Banana", "A yellow fruit", seller, today, nextWeek);
		auctionService.create(item);

		Category category = new Category("Fruits");
		category.addItem(item);
		auctionService.create(category);


		EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
				EntityManagerFactory.class).get();
		EntityManager em = entityManagerFactory.createEntityManager();
		em.getTransaction().begin();
		final Item merged = em.merge(item);
		for (Category cat : merged.getCategories()) {
			cat.removeItem(merged);
		}
		em.remove(merged);
		em.getTransaction().commit();
		em.close();

	}

	@Test
	public void inheritanceTest(){
		UserService userService = weldContainer.instance().select(UserService.class).get();
		AuctionService auctionService = weldContainer.instance().select(AuctionService.class).get();

		User seller = new User("Credit card millionaire", "..", "mm", "p", "mill@example.com");
		CreditCard creditCard = new CreditCard("mm", seller, "1234", CreditCardType.VISA, "02", "11");
		seller.addBillingDetails(creditCard);
		userService.save(seller);

		EntityManagerFactory entityManagerFactory = weldContainer.instance().select(
				EntityManagerFactory.class).get();
		EntityManager em = entityManagerFactory.createEntityManager();
		final List<BillingDetails> resultList = em.createQuery("select b from BillingDetails  b where b.user.id= :userId",
				BillingDetails.class).setParameter("userId", seller.getId()).getResultList();
		Assert.assertEquals(1, resultList.size());

	}


}
